<?php
/**
 * User: nester_a
 * Date: 22.07.14
 */

namespace LMS\Tests\Database;

use LMS\Libraries\ServiceLocator;
use LMS\Tests\Libs\DatabaseTestCase;
use LMS\Tests\Data\Tables;

/**
 * TODO: add tests for \InvalidArgumentException
 *
 * Class QueryTest
 * @package LMS\Tests\Database
 */
class QueryTest extends DatabaseTestCase
{
	/**
	 * @var \LMS\Libraries\Database\Query
	 */
	private $query;

	/**
	 * @var \LMS\Libraries\Database\Driver
	 */
	private $db;

	private $needRunSQL = true;

	public function setUp()
	{
		parent::setUp();

		$this->db = ServiceLocator::get('db');

		ServiceLocator::setMocks(array(
			'db' => $this->db
		));

		$this->query = new \LMS\Libraries\Database\Query($this->dbo->getQuery(true), $this->db );
	}



	/**
	 *
	 */
	public function testExceptionFrom()
	{
		$this->needRunSQL = false;
		$this->setExpectedException('InvalidArgumentException');
		$this->query->from('some table name');
	}

	/**
	 *
	 */
	public function testExceptionUpdate()
	{
		$this->needRunSQL = false;
		$this->setExpectedException('InvalidArgumentException');
		$this->query->update('some table name');
	}

	/**
	 *
	 */
	public function testExceptionDelete()
	{
		$this->needRunSQL = false;
		$this->setExpectedException('InvalidArgumentException');
		$this->query->delete('some table name');
	}

	/**
	 *
	 */
	public function testExceptionSelect()
	{
		$this->needRunSQL = false;
		$this->setExpectedException('InvalidArgumentException');
		$this->query->select('somefield');
	}

	/**
	 *
	 */
	public function testExceptionSelectWithoutTableAlias()
	{
		$table = new Tables\TestTable1($this->dbo);
		$this->needRunSQL = false;
		$this->setExpectedException('InvalidArgumentException');

		$this->query->from($table);
		$this->query->select('id');
	}

	/**
	 *
	 */
	public function testExceptionWhere()
	{
		$this->needRunSQL = false;
		$this->setExpectedException('InvalidArgumentException');
		$this->query->where('somefield = 1');
	}

	/**
	 *
	 */
	public function testGetTable()
	{
		$table = new Tables\TestTable1($this->dbo);
		$this->query->from($table);
		$table2 = $this->query->getTable();
		$this->assertEquals($table, $table2);
		$this->needRunSQL = false;
	}

	/**
	 * @depends testGetTable
	 */
	public function testSetTable()
	{
		$table = new Tables\TestTable1($this->dbo);
		$this->query->setTable($table);
		$table2 = $this->query->getTable();
		$this->assertEquals($table, $table2);
		$this->needRunSQL = false;
	}

	/**]
	 *
	 */
	public function testSelectAll()
	{
		$table = new Tables\TestTable1($this->dbo);
		$alias = $table->getTableAlias();

		$this->query->from($table);
		$this->query->select(array($alias.'.*'));

		$sql = $this->_formatSQL((string)$this->query);

		$this->assertContains('SELECT table_1.id as id, table_1.key_id as key_id, table_1.name as table_name', $sql, 'no SELECT');
	}

	/**
	 *
	 */
	public function testSelect()
	{
		$table = new Tables\TestTable1($this->dbo);
		$alias = $table->getTableAlias();

		$this->query->from($table);
		$this->query->select(array($alias.'.child1.child2.child1.child2.id'));

		$sql = $this->_formatSQL((string)$this->query);

		$this->assertContains('SELECT table_1_child1_child2_child1_child2.id', $sql, 'no SELECT');
		$this->assertContains('FROM #__lms_table1 as table_1', $sql, 'no FROM with right table');
		$this->assertContains('JOIN #__lms_table2 table_1_child1', $sql, 'no JOIN for table #__lms_table2');
		$this->assertContains('JOIN #__lms_table4 table_1_child1_child2', $sql, 'no JOIN for table #__lms_table4');
		$this->assertContains('JOIN #__lms_table1 table_1_child1_child2_child1', $sql, 'no JOIN for table #__lms_table1');
		$this->assertContains('JOIN #__lms_table3 table_1_child1_child2_child1_child2', $sql, 'no JOIN for table #__lms_table3');
	}

	/**
	 *
	 */
	public function testSelectAlias()
	{
		$table = new Tables\TestTable1($this->dbo);
		$alias = $table->getTableAlias();

		$this->query->from($table);
		$this->query->select(array($alias.'.child1.child2.child1.id'));

		$sql = $this->_formatSQL((string)$this->query);

		$this->assertContains('SELECT table_1_child1_child2_child1.id as id', $sql, 'no SELECT with right alias');
	}

	/**
	 *
	 */
	public function testSelectAliasExists()
	{
		$table = new Tables\TestTable1($this->dbo);
		$alias = $table->getTableAlias();

		$this->query->from($table);
		$this->query->select(array($alias.'.child1.child2.child1.id as some_alias'));

		$sql = $this->_formatSQL((string)$this->query);

		$this->assertContains('SELECT table_1_child1_child2_child1.id as some_alias', $sql, 'no SELECT with right alias');
	}

	/**
	 *
	 */
	public function testSelectWithWhere()
	{
		$table = new Tables\TestTable1($this->dbo);
		$alias = $table->getTableAlias();

		$this->query->from($table);
		$this->query->select('table_1.id');
		$this->query->select($alias.'.child1.child2.id');
		$this->query->where($alias.'.child1.child2.child1.child2.id = 2');
		$this->query->where($alias.'.child1.child2.child1.name = "name"');

		$sql = $this->_formatSQL((string)$this->query);

		$this->assertContains('table_1_child1_child2_child1_child2.id = 2', $sql, 'no WHERE with right condition 1');
		$this->assertContains('table_1_child1_child2_child1.name = "name"', $sql, 'no WHERE with right condition 2');

		$this->assertContains('FROM #__lms_table1 as table_1', $sql, 'no FROM with right table');
		$this->assertEquals(1, substr_count($sql, 'JOIN #__lms_table2'), '#__lms_table2 JOINED more than once');

		$this->assertContains('JOIN #__lms_table2 table_1_child1', $sql, 'no JOIN for table #__lms_table2');
		$this->assertEquals(1, substr_count($sql, 'JOIN #__lms_table2'), '#__lms_table2 JOINED more than once');


		$this->assertContains('JOIN #__lms_table4 table_1_child1_child2', $sql, 'no JOIN for table #__lms_table4');
		$this->assertEquals(1, substr_count($sql, 'JOIN #__lms_table4'), '#__lms_table4 JOINED more than once');


		$this->assertContains('JOIN #__lms_table1 table_1_child1_child2_child1', $sql, 'no JOIN for table #__lms_table1');
		$this->assertEquals(1, substr_count($sql, 'JOIN #__lms_table1'), '#__lms_table1 JOINED more than once');


		$this->assertContains('JOIN #__lms_table3 table_1_child1_child2_child1_child2', $sql, 'no JOIN for table #__lms_table3');
		$this->assertEquals(1, substr_count($sql, 'JOIN #__lms_table3'), '#__lms_table3 JOINED more than once');

	}

	/**
	 *
	 */
	public function testSelectWithSpecialFunctions()
	{
		$table = new Tables\TestTable1($this->dbo);

		$this->query->from($table);
		$this->query->select('COUNT(table_1.id), COUNT(table_1.child1.id), COUNT(table_1.child1.child2.id), COUNT(table_1.child1.child2.child2.id) as count');

		$sql = $this->_formatSQL((string)$this->query);

		$this->assertContains('COUNT(table_1.id)', $sql, 'no 1 COUNT function');
		$this->assertContains('COUNT(table_1_child1.id)', $sql, 'no 2 COUNT function');
		$this->assertContains('COUNT(table_1_child1_child2.id)', $sql, 'no 3 COUNT function');
		$this->assertContains('COUNT(table_1_child1_child2_child2.id) as count', $sql, 'no 4 COUNT function');

		$this->assertContains('FROM #__lms_table1 as table_1', $sql, 'no FROM with right table');
		$this->assertContains('FROM #__lms_table1 as table_1', $sql, 'no FROM with right table');
		$this->assertContains('FROM #__lms_table1 as table_1', $sql, 'no FROM with right table');
		$this->assertContains('JOIN #__lms_table2 table_1_child1', $sql, 'no JOIN for table #__lms_table2');
		$this->assertContains('JOIN #__lms_table4 table_1_child1_child2', $sql, 'no JOIN for table #__lms_table4');
		$this->assertContains('JOIN #__lms_table2 table_1_child1_child2_child2', $sql, 'no JOIN for table #__lms_table2 (2)');
	}

	/**
	 *
	 */
	public function testDelete()
	{
		$table = new Tables\TestTable1($this->dbo);

		$this->query->delete($table);
		$this->query->where(array('id = 1'));
		$this->query->where(array('key_id = 2'));

		$sql = $this->_formatSQL((string)$this->query);

		$this->assertContains('DELETE FROM #__lms_table1', $sql, 'no DELETE');
		$this->assertContains('WHERE id = 1 AND key_id = 2', $sql, 'no WHERE with right conditions');
	}

	/**
	 *
	 */
	public function testUpdate()
	{
		$table = new Tables\TestTable1($this->dbo);

		$this->query->update($table);
		$this->query->set(array('name = \'newname\''));
		$this->query->set(array('key_id = 2'));

		$this->query->where(array('id = 1'));

		$sql = $this->_formatSQL((string)$this->query);

		$this->assertContains('UPDATE #__lms_table1', $sql, 'no DELETE');
		$this->assertContains('SET name = \'newname\' , key_id = 2', $sql, 'no SET with right fields');
		$this->assertContains('WHERE id = 1', $sql, 'no WHERE clause');
	}

	/**
	 *
	 */
	public function testSetLimit()
	{
		$table = new Tables\TestTable1($this->dbo);
		$alias = $table->getTableAlias();

		$this->query->from($table);
		$this->query->select(array($alias.'.name'));
		$this->query->setLimit(10, 20);

		$sql = $this->_formatSQL((string)$this->query);

		$this->assertContains('LIMIT 20, 10', $sql, 'no LIMIT');

	}

	protected function tearDown()
	{
		if($this->needRunSQL){
			$this->db->setQuery($this->query);
			$this->db->query();
		}
		parent::tearDown();
	}


	private function _formatSQL($sql)
	{
		$sql = str_replace("\r\n", " ", $sql);
		$sql = str_replace("\n", " ", $sql);
		return preg_replace('/\s+/', " ", $sql);
	}
}